# EMReady

## Improvement of cryo-EM maps for protein and nucleic acid using heterogeneity-aware deep learning

<a href="#"><img src="https://img.shields.io/badge/Linux-Tested-yellow?logo=Linux&style=for-the-badge" /></a>
<a href="#"><img src="https://img.shields.io/badge/Windows-Tested-white?logo=Windows&style=for-the-badge" /></a>

<a href="https://pytorch.org/"><img src="https://img.shields.io/badge/PyTorch-v2.0-red.svg?logo=PyTorch&style=for-the-badge" /></a>
<a href="https://developer.nvidia.com/cuda-toolkit"><img src="https://img.shields.io/badge/CUDA-v11.7-green.svg?logo=Nvidia&style=for-the-badge" /></a>
<a href="https://python.org"><img src="https://img.shields.io/badge/python-v3.9-blue.svg?logo=python&style=for-the-badge" /></a>

<a href="https://www.gnu.org/licenses/gpl-3.0.en.html#license-text"><img src="https://img.shields.io/badge/GPL%20License-v3.0-purple.svg?logo=GNU&style=for-the-badge" /></a>

<img src='assets/main.png' width='800'>

Copyright (C) 2024  Hong Cao, Jiahua He, Tao Li, Sheng-You Huang and Huazhong University of Science and Technology

If you have any questions regarding this program, please don't hesitate to contact us via [huangsy@hust.edu.cn](mailto:huangsy@hust.edu.cn)

</br>

:bangbang:Citation of the following references should be included in any publication that uses the data or results generated by EMReady:

> Hong Cao*, Jiahua He*, Tao Li, and Sheng-You Huang. Improvement of cryo-EM maps for protein and nucleic acid using heterogeneity-aware deep learning. *In submittion* (2024).

> Jiahua He, Tao Li, and Sheng-You Huang. [Improvement of cryo-EM maps by simultaneous local and non-local deep learning](https://doi.org/10.1038/s41467-023-39031-1). *Nature Communications* (2023).
</br>

<!-- 
---

## Introduction

To be finished...
</br> -->

---

## List of contents
| *File name* | *Description* |
| :---- | :---- |
| **EMReady.sh** | The main program wrapped in a shell script to run EMReady |
| **environment.yml** | Required packages for Python virtual environment of EMReady |
| **frn.py** | Pytorch implementation of Filter Response Normalization Layer |
| **interp3d.f90** | Fortran source code for the interpolation of EM grid |
| **pred.py** | Python script of EMReady |
| **scunet.py** | Pytorch implementation of 3D Swin-Conv-UNet |
| **utils.py** | Python utilities |
| **interp_back.py** | Python script for inverse interpolation |
| **model_state_dicts/model_grid_size_1.0.pth** | The trained model with 1.0 Angstrom grid size |
| **model_state_dicts/model_grid_size_0.5.pth** | The trained model with 0.5 Angstrom grid size |
</br>

---

## Install EMReady

### 1. Installation of required online python packages using conda

     conda env create -f environment.yml

This command will create a [Python](https://www.python.org) (ver. 3.9.12) virtual environment named "**emready_env**" 
and install all the required packages automatically using [conda](https://conda.org/), e.g., 
[Anaconda](https://www.anaconda.com/), [Miniconda](https://docs.anaconda.com/miniconda/), [Miniforge](https://github.com/conda-forge/miniforge) (recommended), etc.


#### Detailed requirements of python packages

| *Package name* | *Recommended version* |
| :---- | :---- |
| [pytorch](https://pytorch.org) | 2.0.0 |
| [pytorch-cuda](https://pytorch.org) | 11.7 |
| [einops](https://github.com/arogozhnikov/einops) | 0.6.1 |
| [timm](https://github.com/huggingface/pytorch-image-models) | 0.9.2 |
| [biopython](https://biopython.org/) | 1.81 |
| [numpy](https://www.numpy.org) | 1.24.2 |
| [mrcfile](https://github.com/ccpem/mrcfile) | 1.3.0 |
| [tqdm](https://github.com/tqdm/tqdm) | 4.60.0 |
</br>

### 2. Build the interpolation program "interp3d.f90" as a python package using f2py

:exclamation:The version of $\rm \color{magenta} f2py$ should match the version of Python in EMReady's conda environment:

     conda activate emready_env
</br>

     f2py -c ./interp3d.f90 -m interp3d

This command will generate an ELF file with name like "**interp3d.cpython-\*.so**". Please keep "**interp3d.cpython-\*.so**" with all python scripts in the same directory (the root directory of EMReady).

Fortran compiler (e.g.， [gfortran](https://gcc.gnu.org/fortran/), [ifort](https://www.intel.com/content/www/us/en/developer/tools/oneapi/fortran-compiler.html), etc) is required to run $\rm \color{magenta} f2py$. For Linux systems with Debian package management (e.g., Debian, Ubuntu), **gfortran** can be easily installed via:

     sudo apt-get install gfortran

Alternatively, users can also install **gfortran** via conda (in the same conda environment of EMReady):

     conda install -c conda-forge gfortran==11.4

If the fortran compiler is not in PATH, users can manually specify the path of their fortran compiler, for example:

     f2py -c ./interp3d.f90 -m interp3d --fcompiler=gnu95 --f77exec=/path/to/gfortran --f90exec=/path/to/gfortran

For more information about $\rm \color{magenta} f2py$, please refer to the [official documentation](https://numpy.org/doc/stable/f2py/).

</br>

### 3. Set user-specific variables in "EMReady.sh"

**a**. Set "**EMReady_home**" to the root directory of EMReady. For example, if EMReady is unzipped to "/home/hcao/EMReady", set:

     EMReady_home="/home/hcao/EMReady"

**b**. Set "**activate**" to path of conda activate script. For example, if [anaconda](https://www.anaconda.com/) is located at "/home/hcao/anaconda3/", set:

     activate="/home/hcao/anaconda3/bin/activate"

**c**. Set "**EMReady_env**" to name of the conda environment that have all the required packages installed. 
By default, the conda environment for EMReady is named as "emready_env", so we set:

     EMReady_env="emready_env"
</br>

---

## Run EMReady

### Usage:

     ./EMReady.sh in_map.mrc out_map.mrc [Options]

#### Required arguments:

     in_map.mrc:    File name of the input EM density map in MRC2014 format.

     out_map.mrc:   File name of the output EMReady-processed density map.

### Options:

| *Option* | *Description* | *Default* |
| :---- | :---- | :---- |
| -g | ID(s) of GPU devices to be used, e.g., *0* for GPU #0, and *2,3,6* for GPUs #2, #3, and #6 | 0 |
| -s | The stride of the sliding window to cut the input map into overlapping boxes | 12 |
| -b | Number of input boxes in one batch | 10 |
| -m | Input mask map in MRC2014 format | - |
| -c | The contour threshold to binarize the mask map | 0.0 |
| -p | Input structure in PDB or mmCIF format to be used as the mask | - |
| -r | Zone radius of the mask around the input structure in Angstrom | 4.0 |
| -mo | Filename of the output binary mask map | - | 
| --use_cpu | Run EMReady on CPU instead of GPU | - |
| --inverse | Inverse the mask | - |
| --interp_back | Interpolate the grid size of the processed map back to the original size | - |

### Notes:

1. Users can adjust the stride of the sliding window by option $\rm \color{orange} \text{-}s$ to control the number of overlapping boxes to be calculated.
The larger the stride is, the fewer the overlapping boxes need to be calculated, and vice versa
The size of the overlapping boxes is fixed to 48×48×48, so the maximum value of stride is 48.
However, the stride should not be too large, otherwise some inconsistencies among sliding boxes will be introduced to the processed map.
In most cases, the default value (12) is recommended. For larger density maps, a stride value of 24 is a decent choice.

2. By default, EMReady will run on GPU(s). Users can adjust the batch size according to the available VRAM of their GPU by option $\rm \color{orange} \text{-}b$.
Empirically, an NVIDIA A100 with 40 GB VRAM can afford a batch size of about 200. 
Users can also run EMReady on the cpu using option $\rm \color{orange} \text{--}use\underline{ }cpu$.
But this may take extremely long time for large density maps.

3. Users can provide a mask map in MRC2014 format by option $\rm \color{orange} \text{-}m$, where the contour threshold to binarize the mask map can be specified by option $\rm \color{orange} \text{-}c$.
Alternatively, the mask map can also be generated from a given input structure by option $\rm \color{orange} \text{-}p$, where the mask will be generated around the heavy atoms within a radius specified by option $\rm \color{orange} \text{-}r$. Users can also inversely apply the mask by option $\rm \color{orange} \text{--}inverse$. In addition, the binary mask map can be saved by option $\rm \color{orange} \text{-}mo$.

4. There are two EMReady models trained at two different grid sizes: 0.5 Angstrom and 1.0 Angstrom. Depending on the grid size of the input map, the corresponding model will be automatically selected. Specifically, if the grid size of the input map is less than 1.0 Angstrom, the model with 0.5 Angstrom grid size will be used; otherwise, the model with 1.0 Angstrom grid size will be used.

5. During the processing, the grid size of the input map will be interpolated to 0.5 or 1.0 Angstrom, depending on the model used. By default, the grid size of the output processed map is 0.5 or 1.0 Angstrom.
However, users can choose to interpolate the grid size of the output processed map back to the original size by option $\rm \color{orange} \text{--}interp\underline{ }back$, while the processed map at EMReady model's grid size (0.5 or 1.0 Angstrom) will also be saved in the same directory as **\*_grid_size_0.5.mrc** or **\*_grid_size_1.0.mrc**.
</br>

---

## Examples

### EMD-6551
Clike here to download: the [input density map](http://huanglab.phys.hust.edu.cn/EMReady/examples/emd_6551.map.gz) and the [output processed map](http://huanglab.phys.hust.edu.cn/EMReady/examples/6551_EMReady.mrc.gz)

<img src="assets/emd-6551.jpg" width="800">
</br>

### EMD-9105
Clike here to download: the [input density map](http://huanglab.phys.hust.edu.cn/EMReady/examples/emd_9105.map.gz) and the [output processed map](http://huanglab.phys.hust.edu.cn/EMReady/examples/9105_EMReady.mrc.gz)

<img src="assets/emd-9105.jpg" width="800">
